"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const better_path_join_1 = require("better-path-join");
const status_1 = require("@ts-schema-autogen/status");
const instruction_1 = require("./instruction");
const output_descriptor_1 = require("./output-descriptor");
const load_config_1 = require("./load-config");
/** Determine output files from instruction and delete them */
function cleanUnit(param) {
    const { directory } = param;
    const { unlink } = param.fsx;
    const join = better_path_join_1.createJoinFunction(param.path);
    function handleOutputDescriptor(desc) {
        const filename = join(directory, desc.filename);
        const execute = () => unlink(filename);
        return { filename, execute };
    }
    const result = instruction_1.listSymbolInstruction(param.instruction)
        .map(instruction => output_descriptor_1.ensureOutputDescriptorArray(instruction.output))
        .flat()
        .map(handleOutputDescriptor);
    return result;
}
exports.cleanUnit = cleanUnit;
/** Determine output files from multiple configs and delete them */
async function clean(param) {
    const { fsx, path } = param;
    const configLoader = new load_config_1.ConfigLoader(param);
    const deleteFunctions = [];
    const errors = [];
    for await (const configFile of param.configFiles) {
        const configResult = await configLoader.loadConfig(configFile);
        if (configResult.code) {
            errors.push(configResult);
            continue;
        }
        const removalInstructions = cleanUnit({
            fsx,
            path,
            directory: path.dirname(configFile),
            instruction: configResult.value.instruction
        });
        for (const { filename, execute } of removalInstructions) {
            const fn = () => execute().catch(error => {
                errors.push(new status_1.FileRemovalFailure(filename, error));
            });
            deleteFunctions.push(fn);
        }
    }
    // Problem: With eager deletion, `fast-traverse` calls `stat` upon files that no longer exist
    // Solution: Use lazy method of deletion
    // TODO: Add test to detect this problem
    // TODO: Fix fastTraverse
    await Promise.all(deleteFunctions.map(fn => fn()));
    return status_1.MultipleFailures.maybe(errors) || new status_1.Success(undefined);
}
exports.clean = clean;
//# sourceMappingURL=clean.js.map