import { createJoinFunction } from "better-path-join/index.mjs";
import { FileRemovalFailure, MultipleFailures, Success } from "@ts-schema-autogen/status/index.mjs";
import { listSymbolInstruction } from "./instruction.mjs";
import { ensureOutputDescriptorArray } from "./output-descriptor.mjs";
import { ConfigLoader } from "./load-config.mjs";
/** Determine output files from instruction and delete them */

export function cleanUnit(param) {
  const {
    directory
  } = param;
  const {
    unlink
  } = param.fsx;
  const join = createJoinFunction(param.path);

  function handleOutputDescriptor(desc) {
    const filename = join(directory, desc.filename);

    const execute = () => unlink(filename);

    return {
      filename,
      execute
    };
  }

  const result = listSymbolInstruction(param.instruction).map(instruction => ensureOutputDescriptorArray(instruction.output)).flat().map(handleOutputDescriptor);
  return result;
}
/** Determine output files from multiple configs and delete them */

export async function clean(param) {
  const {
    fsx,
    path
  } = param;
  const configLoader = new ConfigLoader(param);
  const deleteFunctions = [];
  const errors = [];

  for await (const configFile of param.configFiles) {
    const configResult = await configLoader.loadConfig(configFile);

    if (configResult.code) {
      errors.push(configResult);
      continue;
    }

    const removalInstructions = cleanUnit({
      fsx,
      path,
      directory: path.dirname(configFile),
      instruction: configResult.value.instruction
    });

    for (const {
      filename,
      execute
    } of removalInstructions) {
      const fn = () => execute().catch(error => {
        errors.push(new FileRemovalFailure(filename, error));
      });

      deleteFunctions.push(fn);
    }
  } // Problem: With eager deletion, `fast-traverse` calls `stat` upon files that no longer exist
  // Solution: Use lazy method of deletion
  // TODO: Add test to detect this problem
  // TODO: Fix fastTraverse


  await Promise.all(deleteFunctions.map(fn => fn()));
  return MultipleFailures.maybe(errors) || new Success(undefined);
} //# sourceMappingURL=clean.js.map