import { ValidationError } from '@ts-schema-autogen/validate';
import { ConfigParseError, ConfigLoader } from './load-config';
import { Program, Definition, Settings, Instruction, SymbolInstruction, OutputDescriptor, TJS, Path } from '@ts-schema-autogen/types';
import { Failure, MultipleFailures, OutputFileConflict, FileWritingFailure, FileReadingFailure, TextParsingFailure, CircularReference, OutdatedFile, MissingFileParser, GeneratorConstructingFailure, UnsatisfiedSchema, Success } from '@ts-schema-autogen/status';
/** Instruction created by {@link generateUnit} and used by {@link SchemaWriter} to write schema files */
export interface FileWritingInstruction<Definition> {
    readonly schema: Definition;
    readonly instruction: SymbolInstruction;
}
/** Create a {@link FileWritingInstruction} from an `Instruction` */
export declare function generateUnit<Prog = Program, Def = Definition>(param: generateUnit.Param<Prog, Def>): generateUnit.Return<Def>;
export declare namespace generateUnit {
    interface Param<Program, Definition> {
        /** `typescript-json-schema` module to convert TypeScript types into JSON schemas */
        readonly tjs: TJS.Mod<Program, Definition>;
        /** Instruction that tells types, input, and output */
        readonly instruction: Instruction;
        /** Path resolver */
        readonly resolvePath: PathResolver;
    }
    interface PathResolver {
        /**
         * Resolve output path
         * @param output Output path from instruction (relative to config file)
         * @returns Resolved path to pass to writer function (relative to working directory)
         */
        (output: string): string;
    }
    type Return<Definition> = GeneratorConstructingFailure<Settings | undefined> | Success<Iterable<FileWritingInstruction<Definition>>>;
}
export declare const serialize: (schema: any, { indent }: OutputDescriptor) => string;
/** Call a function on a list of {@link FileWritingInstruction} */
export declare function processWriteInstructions<ActFailure extends Failure<any>, ActRejection extends Failure<any>>(param: processWriteInstructions.Param<ActFailure, ActRejection>): Promise<processWriteInstructions.Return<ActFailure, ActRejection>>;
export declare namespace processWriteInstructions {
    interface Param<Failure, Rejection> {
        /** Function to be called on each {@link FileWritingInstruction} */
        readonly act: Act<Failure>;
        /** Function to be called on `act`'s rejection */
        readonly onreject: OnReject<Rejection>;
        /** List of writing instructions */
        readonly instruction: Iterable<FileWritingInstruction<any>>;
    }
    interface Act<Failure> {
        /**
         * @param filename Path to a file
         * @param content Content to write or compare to the file
         */
        (filename: string, content: string): Promise<Failure | void>;
    }
    interface OnReject<Rejection> {
        /**
         * @param filename Path to the file that caused rejection
         * @param reason Rejection reason
         */
        (filename: string, reason: any): Rejection;
    }
    type Return<ActFailure extends Failure<any>, ActRejection extends Failure<any>> = OutputFileConflict | MultipleFailures.Maybe<ActFailure | ActRejection> | Success<void>;
}
/** Write schema files according to one or multiple configs */
export declare class SchemaWriter<Prog = Program, Def = Definition> {
    private readonly param;
    constructor(param: SchemaWriter.ConstructorParam<Prog, Def>);
    private readonly loader;
    private static joinCfgRes;
    /**
     * Write schemas according to one config file
     * @param configPath Path to the config file
     * @param path Path module
     */
    singleConfig(configPath: string, path: Path.Mod): Promise<SchemaWriter.SingleConfigReturn<Def>>;
    private mayWriteSchemas;
    /**
     * Write schemas according to multiple config files
     * @param configPaths List of paths to config files
     */
    writeSchemas(configPaths: readonly string[]): Promise<SchemaWriter.WriteSchemaReturn<Def>>;
    /**
     * Test if schema files referred to by config files are up-to-date
     * @param configPaths List of paths to config files
     */
    testSchemas(configPaths: readonly string[]): Promise<SchemaWriter.TestSchemaReturn<Def>>;
}
export declare namespace SchemaWriter {
    export interface ConstructorParam<Program, Definition> extends ConfigLoader.ConstructorParam {
        /** `typescript-json-schema` module to convert TypeScript types into JSON schemas */
        readonly tjs: TJS.Mod<Program, Definition>;
    }
    export type SingleConfigReturn<Definition> = FileReadingFailure | FileWritingFailure | TextParsingFailure<ConfigParseError> | CircularReference | OutputFileConflict | MissingFileParser | GeneratorConstructingFailure<Settings | undefined> | UnsatisfiedSchema<ValidationError> | Success<Iterable<FileWritingInstruction<Definition>>>;
    type SingleConfigFailure<Definition> = Exclude<SingleConfigReturn<Definition>, Success<any>>;
    type WriteTestSchemaReturn<Extra extends Failure<any>, Definition> = MultipleFailures.Maybe<SingleConfigFailure<Definition> | Extra> | OutputFileConflict | Success<void>;
    export type WriteSchemaReturn<Definition> = WriteTestSchemaReturn<FileWritingFailure, Definition>;
    export type TestSchemaReturn<Definition> = WriteTestSchemaReturn<OutdatedFile | FileReadingFailure, Definition>;
    export {};
}
