"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const object_1 = require("@tsfun/object");
const utils_1 = require("@ts-schema-autogen/utils");
const instruction_1 = require("./instruction");
const output_descriptor_1 = require("./output-descriptor");
const load_config_1 = require("./load-config");
const status_1 = require("@ts-schema-autogen/status");
/** Create a {@link FileWritingInstruction} from an `Instruction` */
function generateUnit(param) {
    var _a;
    const { tjs, instruction, resolvePath } = param;
    const { buildGenerator, getProgramFromFiles } = tjs;
    const input = utils_1.ensureArray((_a = instruction.input) !== null && _a !== void 0 ? _a : []).map(path => resolvePath(path));
    const program = getProgramFromFiles(input, instruction.compilerOptions);
    const settings = instruction.schemaSettings;
    const generator = buildGenerator(program, settings);
    if (!generator)
        return new status_1.GeneratorConstructingFailure({ input, settings });
    function* generate() {
        for (const symbolInstruction of instruction_1.listSymbolInstruction(instruction)) {
            const schema = generator.getSchemaForSymbol(symbolInstruction.symbol);
            const output = output_descriptor_1.ensureOutputDescriptorArray(symbolInstruction.output)
                .map(item => object_1.addProperty(item, 'filename', resolvePath(item.filename)));
            const instruction = object_1.addProperty(symbolInstruction, 'output', output);
            yield { instruction, schema };
        }
    }
    return new status_1.Success({
        [Symbol.iterator]: generate
    });
}
exports.generateUnit = generateUnit;
exports.serialize = (schema, { indent }) => JSON.stringify(schema, undefined, utils_1.getIndent(indent)) + '\n';
/** Call a function on a list of {@link FileWritingInstruction} */
async function processWriteInstructions(param) {
    const { act, onreject, instruction } = param;
    const duplicationCheckingArray = [];
    const duplicationMap = new Map();
    const actFuncs = [];
    const actErrors = [];
    for (const { schema, instruction: { output } } of instruction) {
        const descriptors = output_descriptor_1.ensureOutputDescriptorArray(output);
        for (const desc of descriptors) {
            const { filename } = desc;
            const duplicatedFiles = duplicationCheckingArray.filter(x => x.filename === filename);
            if (duplicatedFiles.length) {
                if (duplicationMap.has(filename)) {
                    duplicationMap.get(filename).push(desc);
                }
                else {
                    duplicationMap.set(filename, [...duplicatedFiles, desc]);
                }
                continue;
            }
            duplicationCheckingArray.push(desc);
            actFuncs.push(() => act(filename, exports.serialize(schema, desc))
                .then(failure => failure && actErrors.push(failure))
                .catch(error => actErrors.push(onreject(filename, error))));
        }
    }
    if (duplicationMap.size)
        return new status_1.OutputFileConflict(duplicationMap);
    await Promise.all(actFuncs.map(fn => fn()));
    return status_1.MultipleFailures.maybe(actErrors) || new status_1.Success(undefined);
}
exports.processWriteInstructions = processWriteInstructions;
/** Write schema files according to one or multiple configs */
class SchemaWriter {
    constructor(param) {
        this.param = param;
        this.loader = new load_config_1.ConfigLoader(this.param);
    }
    static joinCfgRes(list) {
        const errors = [];
        let instruction = [];
        for (const item of list) {
            if (item.code) {
                errors.push(item);
            }
            else {
                instruction = utils_1.concatIterable(instruction, item.value);
            }
        }
        return { errors, instruction };
    }
    /**
     * Write schemas according to one config file
     * @param configPath Path to the config file
     * @param path Path module
     */
    async singleConfig(configPath, path) {
        const config = await this.loader.loadConfig(configPath);
        if (config.code)
            return config;
        const { join } = path;
        const dirname = path.dirname(configPath);
        const writeInstruction = generateUnit(object_1.objectExtends(this.param, {
            instruction: config.value.instruction,
            resolvePath: (filename) => join(dirname, filename)
        }));
        if (writeInstruction.code)
            return writeInstruction;
        return new status_1.Success(writeInstruction.value);
    }
    async mayWriteSchemas(act, onreject, configPaths, path) {
        if (!this.param.parsers.length)
            return new status_1.MissingFileParser();
        const { errors, instruction } = SchemaWriter.joinCfgRes(await Promise.all(configPaths.map(x => this.singleConfig(x, path))));
        return status_1.MultipleFailures.maybe(errors) ||
            processWriteInstructions({ act, onreject, instruction });
    }
    /**
     * Write schemas according to multiple config files
     * @param configPaths List of paths to config files
     */
    writeSchemas(configPaths) {
        const { outputFile } = this.param.fsx;
        const act = (filename, content) => outputFile(filename, content);
        const onreject = (filename, reason) => new status_1.FileWritingFailure(filename, reason);
        return this.mayWriteSchemas(act, onreject, configPaths, this.param.path);
    }
    /**
     * Test if schema files referred to by config files are up-to-date
     * @param configPaths List of paths to config files
     */
    testSchemas(configPaths) {
        const { pathExists, readFile } = this.param.fsx;
        async function act(filename, expectedContent) {
            const exists = pathExists(filename);
            const content = readFile(filename, 'utf8');
            const receivedContent = (await exists) ? await content : null;
            content.catch(() => undefined); // silence error/warning of unhandled rejection
            if (expectedContent !== receivedContent) {
                return new status_1.OutdatedFile(filename, { expectedContent, receivedContent });
            }
        }
        const onreject = (filename, reason) => new status_1.FileReadingFailure(filename, reason);
        return this.mayWriteSchemas(act, onreject, configPaths, this.param.path);
    }
}
exports.SchemaWriter = SchemaWriter;
//# sourceMappingURL=generate.js.map