import { addProperty, objectExtends } from "@tsfun/object/index.mjs";
import { ensureArray, concatIterable, getIndent } from "@ts-schema-autogen/utils/index.mjs";
import { listSymbolInstruction } from "./instruction.mjs";
import { ensureOutputDescriptorArray } from "./output-descriptor.mjs";
import { ConfigLoader } from "./load-config.mjs";
import { MultipleFailures, OutputFileConflict, FileWritingFailure, FileReadingFailure, OutdatedFile, MissingFileParser, GeneratorConstructingFailure, Success } from "@ts-schema-autogen/status/index.mjs";
/** Create a {@link FileWritingInstruction} from an `Instruction` */

export function generateUnit(param) {
  var _a;

  const {
    tjs,
    instruction,
    resolvePath
  } = param;
  const {
    buildGenerator,
    getProgramFromFiles
  } = tjs;
  const input = ensureArray((_a = instruction.input) !== null && _a !== void 0 ? _a : []).map(path => resolvePath(path));
  const program = getProgramFromFiles(input, instruction.compilerOptions);
  const settings = instruction.schemaSettings;
  const generator = buildGenerator(program, settings);
  if (!generator) return new GeneratorConstructingFailure({
    input,
    settings
  });

  function* generate() {
    for (const symbolInstruction of listSymbolInstruction(instruction)) {
      const schema = generator.getSchemaForSymbol(symbolInstruction.symbol);
      const output = ensureOutputDescriptorArray(symbolInstruction.output).map(item => addProperty(item, 'filename', resolvePath(item.filename)));
      const instruction = addProperty(symbolInstruction, 'output', output);
      yield {
        instruction,
        schema
      };
    }
  }

  return new Success({
    [Symbol.iterator]: generate
  });
}
export const serialize = (schema, {
  indent
}) => JSON.stringify(schema, undefined, getIndent(indent)) + '\n';
/** Call a function on a list of {@link FileWritingInstruction} */

export async function processWriteInstructions(param) {
  const {
    act,
    onreject,
    instruction
  } = param;
  const duplicationCheckingArray = [];
  const duplicationMap = new Map();
  const actFuncs = [];
  const actErrors = [];

  for (const {
    schema,
    instruction: {
      output
    }
  } of instruction) {
    const descriptors = ensureOutputDescriptorArray(output);

    for (const desc of descriptors) {
      const {
        filename
      } = desc;
      const duplicatedFiles = duplicationCheckingArray.filter(x => x.filename === filename);

      if (duplicatedFiles.length) {
        if (duplicationMap.has(filename)) {
          duplicationMap.get(filename).push(desc);
        } else {
          duplicationMap.set(filename, [...duplicatedFiles, desc]);
        }

        continue;
      }

      duplicationCheckingArray.push(desc);
      actFuncs.push(() => act(filename, serialize(schema, desc)).then(failure => failure && actErrors.push(failure)).catch(error => actErrors.push(onreject(filename, error))));
    }
  }

  if (duplicationMap.size) return new OutputFileConflict(duplicationMap);
  await Promise.all(actFuncs.map(fn => fn()));
  return MultipleFailures.maybe(actErrors) || new Success(undefined);
}
/** Write schema files according to one or multiple configs */

export class SchemaWriter {
  constructor(param) {
    this.param = param;
    this.loader = new ConfigLoader(this.param);
  }

  static joinCfgRes(list) {
    const errors = [];
    let instruction = [];

    for (const item of list) {
      if (item.code) {
        errors.push(item);
      } else {
        instruction = concatIterable(instruction, item.value);
      }
    }

    return {
      errors,
      instruction
    };
  }
  /**
   * Write schemas according to one config file
   * @param configPath Path to the config file
   * @param path Path module
   */


  async singleConfig(configPath, path) {
    const config = await this.loader.loadConfig(configPath);
    if (config.code) return config;
    const {
      join
    } = path;
    const dirname = path.dirname(configPath);
    const writeInstruction = generateUnit(objectExtends(this.param, {
      instruction: config.value.instruction,
      resolvePath: filename => join(dirname, filename)
    }));
    if (writeInstruction.code) return writeInstruction;
    return new Success(writeInstruction.value);
  }

  async mayWriteSchemas(act, onreject, configPaths, path) {
    if (!this.param.parsers.length) return new MissingFileParser();
    const {
      errors,
      instruction
    } = SchemaWriter.joinCfgRes((await Promise.all(configPaths.map(x => this.singleConfig(x, path)))));
    return MultipleFailures.maybe(errors) || processWriteInstructions({
      act,
      onreject,
      instruction
    });
  }
  /**
   * Write schemas according to multiple config files
   * @param configPaths List of paths to config files
   */


  writeSchemas(configPaths) {
    const {
      outputFile
    } = this.param.fsx;

    const act = (filename, content) => outputFile(filename, content);

    const onreject = (filename, reason) => new FileWritingFailure(filename, reason);

    return this.mayWriteSchemas(act, onreject, configPaths, this.param.path);
  }
  /**
   * Test if schema files referred to by config files are up-to-date
   * @param configPaths List of paths to config files
   */


  testSchemas(configPaths) {
    const {
      pathExists,
      readFile
    } = this.param.fsx;

    async function act(filename, expectedContent) {
      const exists = pathExists(filename);
      const content = readFile(filename, 'utf8');
      const receivedContent = (await exists) ? await content : null;
      content.catch(() => undefined); // silence error/warning of unhandled rejection

      if (expectedContent !== receivedContent) {
        return new OutdatedFile(filename, {
          expectedContent,
          receivedContent
        });
      }
    }

    const onreject = (filename, reason) => new FileReadingFailure(filename, reason);

    return this.mayWriteSchemas(act, onreject, configPaths, this.param.path);
  }

} //# sourceMappingURL=generate.js.map