"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const result_1 = require("@tsfun/result");
const object_1 = require("@tsfun/object");
const better_path_join_1 = require("better-path-join");
const utils_1 = require("@ts-schema-autogen/utils");
const validate_1 = require("@ts-schema-autogen/validate");
const status_1 = require("@ts-schema-autogen/status");
/** Load a config file */
async function loadConfigFile(param) {
    const { filename } = param;
    const readingResult = await param.fsx.readFile(filename, 'utf8').then(result_1.ok, result_1.err);
    if (!readingResult.tag)
        return new status_1.FileReadingFailure(filename, readingResult.error);
    const text = readingResult.value;
    const parseErrors = [];
    for (const parser of param.parsers) {
        if (!parser.testFileName(filename))
            continue;
        const parseResult = parser.parseConfigText(text, filename);
        if (parseResult.tag)
            return new status_1.Success(parseResult.value);
        parseErrors.push({ parser, error: parseResult.error });
    }
    return new status_1.TextParsingFailure(parseErrors);
}
exports.loadConfigFile = loadConfigFile;
/** Merge a config with extensions */
function mergeConfig(primary, ...inherited) {
    const newInstruction = inherited.reduce((instruction, config) => object_1.deepMergeWithPreference(instruction, object_1.omit(config.instruction, MERGE_OMITTED_KEYS), MERGE_CONFLICT_RESOLVER), primary.instruction);
    return object_1.addProperty(primary, 'instruction', newInstruction);
}
exports.mergeConfig = mergeConfig;
const MERGE_OMITTED_KEYS = ['input', 'list', 'output', 'symbol'];
const MERGE_CONFLICT_RESOLVER = ([value]) => value === undefined ? 1 /* Right */ : 0 /* Left */;
/** Load and cache config files */
class ConfigLoader {
    constructor(param) {
        this.param = param;
        this.simpleCache = new Map();
        this.validator = new validate_1.ValidatorFactory();
    }
    async prvLoadConfig(filename, circularGuard) {
        const { simpleCache, param } = this;
        const { dirname } = param.path;
        const join = better_path_join_1.createJoinFunction(param.path);
        if (circularGuard.includes(filename)) {
            return new status_1.CircularReference(circularGuard);
        }
        if (simpleCache.has(filename)) {
            return new status_1.Success(simpleCache.get(filename));
        }
        if (!param.parsers.length)
            return new status_1.MissingFileParser();
        const lcfRet = await loadConfigFile(object_1.addProperty(param, 'filename', filename));
        if (lcfRet.code)
            return lcfRet;
        const validateResult = this.validator.Config(lcfRet.value);
        if (!validateResult.tag)
            return new status_1.UnsatisfiedSchema(filename, validateResult.error);
        let config = validateResult.value;
        if (!config.extends)
            return new status_1.Success(config);
        for (const path of utils_1.ensureArray(config.extends)) {
            const absPath = join(dirname(filename), path);
            const res = await this.prvLoadConfig(absPath, circularGuard.concat(filename));
            if (res.code)
                return res;
            config = mergeConfig(config, res.value);
        }
        simpleCache.set(filename, config);
        return new status_1.Success(config);
    }
    /** Load a config file from cache or from filesystem */
    loadConfig(filename) {
        return this.prvLoadConfig(filename, []);
    }
}
exports.ConfigLoader = ConfigLoader;
//# sourceMappingURL=load-config.js.map