import { ok, err } from "@tsfun/result/index.mjs";
import { addProperty, omit, deepMergeWithPreference } from "@tsfun/object/index.mjs";
import { createJoinFunction } from "better-path-join/index.mjs";
import { ensureArray } from "@ts-schema-autogen/utils/index.mjs";
import { ValidatorFactory } from "@ts-schema-autogen/validate/index.mjs";
import { FileReadingFailure, TextParsingFailure, CircularReference, MissingFileParser, UnsatisfiedSchema, Success } from "@ts-schema-autogen/status/index.mjs";
/** Load a config file */

export async function loadConfigFile(param) {
  const {
    filename
  } = param;
  const readingResult = await param.fsx.readFile(filename, 'utf8').then(ok, err);
  if (!readingResult.tag) return new FileReadingFailure(filename, readingResult.error);
  const text = readingResult.value;
  const parseErrors = [];

  for (const parser of param.parsers) {
    if (!parser.testFileName(filename)) continue;
    const parseResult = parser.parseConfigText(text, filename);
    if (parseResult.tag) return new Success(parseResult.value);
    parseErrors.push({
      parser,
      error: parseResult.error
    });
  }

  return new TextParsingFailure(parseErrors);
}
/** Merge a config with extensions */

export function mergeConfig(primary, ...inherited) {
  const newInstruction = inherited.reduce((instruction, config) => deepMergeWithPreference(instruction, omit(config.instruction, MERGE_OMITTED_KEYS), MERGE_CONFLICT_RESOLVER), primary.instruction);
  return addProperty(primary, 'instruction', newInstruction);
}
const MERGE_OMITTED_KEYS = ['input', 'list', 'output', 'symbol'];

const MERGE_CONFLICT_RESOLVER = ([value]) => value === undefined ? 1
/* Right */
: 0
/* Left */
;
/** Load and cache config files */


export class ConfigLoader {
  constructor(param) {
    this.param = param;
    this.simpleCache = new Map();
    this.validator = new ValidatorFactory();
  }

  async prvLoadConfig(filename, circularGuard) {
    const {
      simpleCache,
      param
    } = this;
    const {
      dirname
    } = param.path;
    const join = createJoinFunction(param.path);

    if (circularGuard.includes(filename)) {
      return new CircularReference(circularGuard);
    }

    if (simpleCache.has(filename)) {
      return new Success(simpleCache.get(filename));
    }

    if (!param.parsers.length) return new MissingFileParser();
    const lcfRet = await loadConfigFile(addProperty(param, 'filename', filename));
    if (lcfRet.code) return lcfRet;
    const validateResult = this.validator.Config(lcfRet.value);
    if (!validateResult.tag) return new UnsatisfiedSchema(filename, validateResult.error);
    let config = validateResult.value;
    if (!config.extends) return new Success(config);

    for (const path of ensureArray(config.extends)) {
      const absPath = join(dirname(filename), path);
      const res = await this.prvLoadConfig(absPath, circularGuard.concat(filename));
      if (res.code) return res;
      config = mergeConfig(config, res.value);
    }

    simpleCache.set(filename, config);
    return new Success(config);
  }
  /** Load a config file from cache or from filesystem */


  loadConfig(filename) {
    return this.prvLoadConfig(filename, []);
  }

} //# sourceMappingURL=load-config.js.map